_ace.GeneralFunctions = {};

_ace.GeneralFunctions.RemoveDuplicates = function(Array){
	var newArray = new Array();
	for(var i = 0; i < Array.length; i++){
		if(newArray.indexOf(Array[i])==-1){
			newArray.push(Array[i]);
		} 
	}
	return newArray;
}

_ace.GeneralFunctions.ElementExists = function(Element){
	return Element.length>0;
}

_ace.GeneralFunctions.RequireNode = function(ObjectNode, Warning){
	if(ObjectNode==undefined){
		throw new Error(Warning);
	}
}

_ace.GeneralFunctions.IdenticalObjects = function( object1, object2 ) {
  if ( object1 === object2 ) return true;
	// if both x and y are null or undefined and exactly the same

  if ( ! ( object1 instanceof Object ) || ! ( object2 instanceof Object ) ) return false;
    // if they are not strictly equal, they both need to be Objects

  if ( object1.constructor !== object2.constructor ) return false;
	// they must have the exact same prototype chain, the closest we can do is
	// test there constructor.

  for ( var p in object1 ) {
    if ( ! object1.hasOwnProperty( p ) ) continue;
	// other properties were tested using x.constructor === y.constructor

    if ( ! object2.hasOwnProperty( p ) ) return false;
	// allows to compare x[ p ] and y[ p ] when set to undefined

    if ( object1[ p ] === object2[ p ] ) continue;
	// if they have the same strict value or identity then they are equal

    if ( typeof( object1[ p ] ) !== "object" ) return false;
     // Numbers, Strings, Functions, Booleans must be strictly equal

    if ( ! _ace.GeneralFunctions.IdenticalObjects( object1[ p ],  object2[ p ] ) ) return false;
   // Objects and Arrays must be tested recursively
  }

  for ( p in object2 ) {
    if ( object2.hasOwnProperty( p ) && ! object1.hasOwnProperty( p ) ) return false;
    // allows x[ p ] to be set to undefined
  }
  return true;
}

_ace.GeneralFunctions.ConvertToString = function(VariableToString){
	var string = '';

	if(typeof VariableToString === 'undefined'){
		return 'null';
	}
	
	if(VariableToString === null){
		return 'null';
	}

	switch(VariableToString.constructor){
		case Function : 
		string += "" + VariableToString;
		break;
		case Boolean : 
		string += VariableToString;
		break;
		case String : 
		string += '"' + VariableToString + '"';
		break;
		case Array : 
		string += '[';
		for(var i = 0; i<VariableToString.length;i++){
			string += _ace.GeneralFunctions.ConvertToString(VariableToString[i]);
			string += ', '
		}
		if(string.length>1){
			string = string.slice(0, -2);
		}
		string += ']';
		break;
		case Object :
			string += '{';
			$.each(VariableToString, function(key, value) {
				string += key + " : " + _ace.GeneralFunctions.ConvertToString(value);
				string += ', '
			});
			if(string.length>1){
				string = string.slice(0, -2);
			}
			string += '}';
		break;
		case Number :
			string += VariableToString;
		break; 
		default : 
			string += '""';
		break;
	}
	return string;
}

_ace.GeneralFunctions.SelectTextInsideId = function(SelectTextInsideIdString){
	if (document.selection) {
		var range = document.body.createTextRange();
		range.moveToElementText(document.getElementById(SelectTextInsideIdString));
		range.select();
	}
	else if (window.getSelection) {
		var range = document.createRange();
		range.selectNode(document.getElementById(SelectTextInsideIdString));
		window.getSelection().addRange(range);
	}
}

_ace.GeneralFunctions.ArrayRemoveAtIndex = function(arrayName,arrayIndex){
	arrayName.splice(arrayIndex,1); 
}

function Rect (top, left, width, height) {
	return rect = {
		FLOAT_top: top,
		FLOAT_left: left,
		FLOAT_width: width,
		FLOAT_height: height
	}
}

function Color (r,g,b,a) {
	return color = {
		INTEGER_r: r,
		INTEGER_g: g,
		INTEGER_b: b,
		INTEGER_a: a
	}
}

function componentToHex(c) {
    var hex = c.toString(16);
    return hex.length == 1 ? "0" + hex : hex;
}

_ace.GeneralFunctions.rgbToHex = function(colorObject) {
	var r = colorObject.INTEGER_r;
	var g = colorObject.INTEGER_g;
	var b = colorObject.INTEGER_b;
    return "#" + componentToHex(r) + componentToHex(g) + componentToHex(b);
}

_ace.GeneralFunctions.colorRgbStringToRgbObject = function(rgbString){
	var explodedRgbString = rgbString.split(",");
	var r = explodedRgbString[0].split("(");
	r = r[r.length-1];
	var tempRgbObject = {
		INTEGER_r : parseFloat(r),
		INTEGER_g : parseFloat(explodedRgbString[1]),
		INTEGER_b : parseFloat(explodedRgbString[2].split(")")[0])
	};
	return tempRgbObject;
}

_ace.GeneralFunctions.colorRgbObjectToRgbString = function(rgbObject){
	return "rgb("+parseInt(rgbObject.INTEGER_r)+", "+parseInt(rgbObject.INTEGER_g)+", "+parseInt(rgbObject.INTEGER_b)+")";
}

_ace.GeneralFunctions.originalImageSize = function(url, callback){
	var image = new Image();
	image.onload = function(){
		callback || callback();
	};
	image.src = url;
	
}

function Execute(functionName, argumentArray){	
	if(argumentArray==undefined) argumentArray=[];
	var returnValue;
	if(typeof functionName == 'function'){
		returnValue = functionName.apply(this, argumentArray);
	} else {
		// console.log("Didn't Execute Function!");
	}
	return returnValue;	
}

_ace.GeneralFunctions.consvertToDisplayText = function(string){
	string = String(string);
	string = string.split("_");
	string = string[string.length-1];
	string = _ace.GeneralFunctions.splitCamelCase(string);
	string = _ace.GeneralFunctions.capitaliseFirstLetter(string);
    return string;
}

_ace.GeneralFunctions.splitCamelCase = function(string){
    return string.replace(/([a-z])([A-Z])/g, '$1 $2').toLowerCase();
}

_ace.GeneralFunctions.capitaliseFirstLetter = function(string){
    return string.charAt(0).toUpperCase() + string.slice(1);
}

_ace.GeneralFunctions.sortNumericOrderAssending = function(a, b){ 
	return (a-b); 
}

_ace.GeneralFunctions.sortNumericOrderDecreasing = function(a, b){ 
	return (b-a); 
}

_ace.GeneralFunctions.convertToFilename = function(theString){
	theString = theString.replace(/[^a-z0-9_\-]/gi, '-').toLowerCase();
	return theString;
}

_ace.GeneralFunctions.capitalizeFirstLetter = function (string) {
        var newString = '';
        string = string.split(' ');
        for(var c=0; c < string.length; c++) {
                newString += string[c].substring(0,1).toUpperCase() + string[c].substring(1,string[c].length);
        }
        return newString;
}

_ace.GeneralFunctions.variableName = function(thisText){
	var allowSpace = false;
	thisText = _ace.GeneralFunctions.capitalizeFirstLetter(thisText);
	var w = "!@#$%^&*()+=[]\\\';,./{}|\":<>?-_";
	var s = 'abcdefghijklmnopqrstuvwxyz0123456789';
	s += 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
	var x = new Array('àáâãäå', 'ç', 'èéêë', 'ìíîï', 'ñ', 'ðóòôõöø', 'ùúûü', 'ýÿ');
	var r = new Array('a', 'c', 'e', 'i', 'n', 'o', 'u', 'y');
	if(allowSpace){
		s = s + ' ';
	}
	var newText = new Array();
	for (i = 0; i < thisText.length; i++){
		thisChar = thisText.charAt(i);
		if(w.indexOf(thisChar) == -1){
			if(s.match(''+thisChar+'')){
				newText[i] = thisChar;
			}else{
				for (j = 0; j < x.length; j++){
					if(x[j].match(thisChar)){
						newText[i] = r[j];
					}
				}
			}
		}
	}
	return newText.join('');
}

_ace.GeneralFunctions.dateToTimeStamp = function (day, month, year) {
	var newDate=month+","+day+","+year;
	return (new Date(newDate).getTime());
}

_ace.GeneralFunctions.ShortenString = function (str, ShortenAfter) {
	str = str.substring(0, ShortenAfter) + "...";
	return str;
}

_ace.GeneralFunctions.mod = function (x, m){
	return (x % m + m) % m;
}

_ace.GeneralFunctions.base64_encode = function(data) {
  // discuss at: http://phpjs.org/functions/base64_encode/
  // original by: Tyler Akins (http://rumkin.com)
  // improved by: Bayron Guevara
  // improved by: Thunder.m
  // improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
  // improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
  // improved by: Rafał Kukawski (http://kukawski.pl)
  // bugfixed by: Pellentesque Malesuada
  // example 1: base64_encode('Kevin van Zonneveld');
  // returns 1: 'S2V2aW4gdmFuIFpvbm5ldmVsZA=='
  // example 2: base64_encode('a');
  // returns 2: 'YQ=='
  // example 3: base64_encode('✓ à la mode');
  // returns 3: '4pyTIMOgIGxhIG1vZGU='

  var b64 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';
  var o1, o2, o3, h1, h2, h3, h4, bits, i = 0,
    ac = 0,
    enc = '',
    tmp_arr = [];

  if (!data) {
    return data;
  }

  data = unescape(encodeURIComponent(data))

  do {
    // pack three octets into four hexets
    o1 = data.charCodeAt(i++);
    o2 = data.charCodeAt(i++);
    o3 = data.charCodeAt(i++);

    bits = o1 << 16 | o2 << 8 | o3;

    h1 = bits >> 18 & 0x3f;
    h2 = bits >> 12 & 0x3f;
    h3 = bits >> 6 & 0x3f;
    h4 = bits & 0x3f;

    // use hexets to index into b64, and append result to encoded string
    tmp_arr[ac++] = b64.charAt(h1) + b64.charAt(h2) + b64.charAt(h3) + b64.charAt(h4);
  } while (i < data.length);

  enc = tmp_arr.join('');

  var r = data.length % 3;

  return (r ? enc.slice(0, r - 3) : enc) + '==='.slice(r || 3);
}

_ace.GeneralFunctions.base64_decode = function(data) {
  // discuss at: http://phpjs.org/functions/base64_decode/
  // original by: Tyler Akins (http://rumkin.com)
  // improved by: Thunder.m
  // improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
  // improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
  // input by: Aman Gupta
  // input by: Brett Zamir (http://brett-zamir.me)
  // bugfixed by: Onno Marsman
  // bugfixed by: Pellentesque Malesuada
  // bugfixed by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
  // example 1: base64_decode('S2V2aW4gdmFuIFpvbm5ldmVsZA==');
  // returns 1: 'Kevin van Zonneveld'
  // example 2: base64_decode('YQ===');
  // returns 2: 'a'
  // example 3: base64_decode('4pyTIMOgIGxhIG1vZGU=');
  // returns 3: '✓ à la mode'

  var b64 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';
  var o1, o2, o3, h1, h2, h3, h4, bits, i = 0,
    ac = 0,
    dec = '',
    tmp_arr = [];

  if (!data) {
    return data;
  }

  data += '';

  do {
    // unpack four hexets into three octets using index points in b64
    h1 = b64.indexOf(data.charAt(i++));
    h2 = b64.indexOf(data.charAt(i++));
    h3 = b64.indexOf(data.charAt(i++));
    h4 = b64.indexOf(data.charAt(i++));

    bits = h1 << 18 | h2 << 12 | h3 << 6 | h4;

    o1 = bits >> 16 & 0xff;
    o2 = bits >> 8 & 0xff;
    o3 = bits & 0xff;

    if (h3 == 64) {
      tmp_arr[ac++] = String.fromCharCode(o1);
    } else if (h4 == 64) {
      tmp_arr[ac++] = String.fromCharCode(o1, o2);
    } else {
      tmp_arr[ac++] = String.fromCharCode(o1, o2, o3);
    }
  } while (i < data.length);

  dec = tmp_arr.join('');

  return decodeURIComponent(escape(dec.replace(/\0+$/, '')));
}

function getBase64FromImageUrl(URL) {
    var img = new Image();
    img.src = URL;
    img.onload = function () {

    var canvas = document.createElement("canvas");
    canvas.width =this.width;
    canvas.height =this.height;

    var ctx = canvas.getContext("2d");
    ctx.drawImage(this, 0, 0);

    var dataURL = canvas.toDataURL("image/png");

    alert(  dataURL.replace(/^data:image\/(png|jpg);base64,/, ""));

    }
}

function HTMLImageElementToBase64(img) {
    var canvas = document.createElement("canvas");
    canvas.width =img.width;
    canvas.height =img.height;
    var ctx = canvas.getContext("2d");
    ctx.drawImage(img, 0, 0);
    var dataURL = canvas.toDataURL("image/png");
	return dataURL;
}








